// Generated by the gRPC C++ plugin.
// If you make any local change, they will be lost.
// source: tensorflow/compiler/xla/xla.proto

#include "tensorflow/compiler/xla/xla.pb.h"
#include "tensorflow/compiler/xla/xla.grpc.pb.h"

#include <grpcpp/impl/codegen/async_stream.h>
#include <grpcpp/impl/codegen/sync_stream.h>
#include <gmock/gmock.h>
namespace xla {


namespace grpc {

} // namespace grpc

} // namespace xla

